/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.common.ui.javafx;

import java.io.IOException;

import org.eclipse.fx.ui.workbench3.FXViewPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import javafx.application.Platform;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.layout.BorderPane;

/**
 * Base class to create eclipse (SWT-based) views using JavaFX GUI elements. This class requires
 * FXML file that defines the GUI elements of the view.
 * 
 * @author diewald
 */
/**
 * This class will be used for the migration to JavaFX, which is work-in-progress. Hence, mark the
 * class for non-removal.
 */
public abstract class AF3FXViewPart extends FXViewPart { // NO_UCD
	/** Holds the relative path to the CSS file defining the Applications appearance. */
	private final static String CSSLOCATION = "/application.css";

	/** For later use, if we need to define different styles. */
	private final String cssLocation;

	/**
	 * Holds the relative location of the FXML JavaXML files. Roots are the source folders of the
	 * (plugin) project.
	 */
	private final String fXMLLocation;

	/** References the stub that defines the FXML file defining the layout and controller. */
	private Class<? extends AF3FXViewPart> viewerClass;

	/**
	 * Constructor. Allows to pass the view's FXML definition file along with style.
	 * 
	 * @param viewerClass
	 *            Stub class that defines the FXML file defining the layout and controller
	 * @param fXMLLocation
	 *            location of the fXML file relative to the source (resource) folders.
	 * @param cssLocation
	 *            style sheet file path for the appearance settings relative to the source
	 *            (resource) folders.
	 * @throws Exception
	 *             if the given FXML file path is not given or if it cannot be found in any of the
	 *             source (resource) folders.
	 */
	private AF3FXViewPart(Class<? extends AF3FXViewPart> viewerClass, String fXMLLocation,
			String cssLocation) throws Exception {
		if(fXMLLocation == null) {
			throw new Exception(
					"The location of the FXML file defining the gui elements was not given," +
							"or it cannot be found at the given location (" + fXMLLocation + ") " +
							"The view will NOT be created.");
		}

		Platform.setImplicitExit(false);
		this.viewerClass = viewerClass;
		this.fXMLLocation = fXMLLocation;
		this.cssLocation = cssLocation;
	}

	/**
	 * Constructor. Allows to pass the view's FXML definition file.
	 * 
	 * @param viewerClass
	 *            Stub class that defines the FXML file defining the layout and controller
	 * @param fXMLLocation
	 *            location of the fXML file relative to the source (resource) folders.
	 * @throws Exception
	 *             if the given FXML file path is not given or if it cannot be found in any of the
	 *             source (resource) folders.
	 */
	public AF3FXViewPart(Class<? extends AF3FXViewPart> viewerClass, String fXMLLocation)
			throws Exception {
		this(viewerClass, fXMLLocation, CSSLOCATION);
	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		if("gtk".equals(System.getProperty("osgi.ws"))) {
			if(!"0".equals(System.getenv("SWT_GTK3"))) {
				org.eclipse.swt.widgets.Label errorLabel =
						new org.eclipse.swt.widgets.Label(parent, SWT.BOLD);
				errorLabel.setText("Cannot show JavaFX view without setting SWT_GTK3 to '0'.\n" +
						"Please relaunch Eclipse using a terminal and 'SWT_GTK3=0 ./eclipse' command.");
				return;
			}
		}
		super.createPartControl(parent);
	}

	/** {@inheritDoc} */
	@Override
	protected Scene createFxScene() {
		BorderPane root = null;
		FXMLLoader loader = new FXMLLoader();
		loader.setClassLoader(viewerClass.getClassLoader());
		loader.setLocation(viewerClass.getResource(fXMLLocation));
		try {
			root = (BorderPane)loader.load();
		} catch(IOException e) {
			throw new RuntimeException("Cannot load the resource located at " + fXMLLocation, e);
		}
		Scene scene = new Scene(root, 400, 400);
		scene.getStylesheets().add(getClass().getResource(cssLocation).toExternalForm());
		return scene;
	}

	/** {@inheritDoc} */
	@Override
	protected void setFxFocus() {
		// Not needed.
	}
}
